@@ -186,7 +186,7 @@ predict.statespacer <- function(object,
   # Forecasting for t = 1 to forecast_period
   for (i in 1:forecast_period) {
 
-    # System matrices for timepoint
+    # System matrices for time point
     if (is.matrix(Z_full)) {
       Z_fc <- Z_full
     } else {
@@ -233,10 +233,8 @@ predict.statespacer <- function(object,
     is.null(level_addvar_list_fc)) {
     tempZ <- matrix(0, p, m)
     tempZ[1:length(Z_padded$level)] <- Z_padded$level
-    result$level <- matrix(0, forecast_period, p)
-    for (i in 1:forecast_period) {
-      result$level[i, ] <- tempZ %*% matrix(a_fc[i, ])
-    }
+    tempZ_t <- t(tempZ)
+    result$level <- a_fc %*% tempZ_t
     Z_padded$level <- tempZ
   }
 
@@ -244,10 +242,8 @@ predict.statespacer <- function(object,
   if (object$function_call$slope_ind && is.null(level_addvar_list_fc)) {
     tempZ <- matrix(0, p, m)
     tempZ[1:length(Z_padded$level)] <- Z_padded$level
-    result$level <- matrix(0, forecast_period, p)
-    for (i in 1:forecast_period) {
-      result$level[i, ] <- tempZ %*% matrix(a_fc[i, ])
-    }
+    tempZ_t <- t(tempZ)
+    result$level <- a_fc %*% tempZ_t
     Z_padded$level <- tempZ
   }
 
@@ -256,10 +252,8 @@ predict.statespacer <- function(object,
     for (s in object$function_call$BSM_vec) {
       tempZ <- matrix(0, p, m)
       tempZ[1:length(Z_padded[[paste0("BSM", s)]])] <- Z_padded[[paste0("BSM", s)]]
-      result[[paste0("BSM", s)]] <- matrix(0, forecast_period, p)
-      for (i in 1:forecast_period) {
-        result[[paste0("BSM", s)]][i, ] <- tempZ %*% matrix(a_fc[i, ])
-      }
+      tempZ_t <- t(tempZ)
+      result[[paste0("BSM", s)]] <- a_fc %*% tempZ_t
       Z_padded[[paste0("BSM", s)]] <- tempZ
     }
   }
@@ -279,10 +273,8 @@ predict.statespacer <- function(object,
   if (!is.null(level_addvar_list_fc) && !object$function_call$slope_ind) {
     tempZ <- matrix(0, p, m)
     tempZ[1:length(Z_padded$level)] <- Z_padded$level
-    result$level <- matrix(0, forecast_period, p)
-    for (i in 1:forecast_period) {
-      result$level[i, ] <- tempZ %*% matrix(a_fc[i, ])
-    }
+    tempZ_t <- t(tempZ)
+    result$level <- a_fc %*% tempZ_t
     Z_padded$level <- tempZ
   }
 
@@ -290,10 +282,8 @@ predict.statespacer <- function(object,
   if (!is.null(level_addvar_list_fc) && object$function_call$slope_ind) {
     tempZ <- matrix(0, p, m)
     tempZ[1:length(Z_padded$level)] <- Z_padded$level
-    result$level <- matrix(0, forecast_period, p)
-    for (i in 1:forecast_period) {
-      result$level[i, ] <- tempZ %*% matrix(a_fc[i, ])
-    }
+    tempZ_t <- t(tempZ)
+    result$level <- a_fc %*% tempZ_t
     Z_padded$level <- tempZ
   }
 
@@ -302,10 +292,8 @@ predict.statespacer <- function(object,
     for (j in seq_along(object$function_call$format_cycle_list)) {
       tempZ <- matrix(0, p, m)
       tempZ[1:length(Z_padded[[paste0("Cycle", j)]])] <- Z_padded[[paste0("Cycle", j)]]
-      result[[paste0("Cycle", j)]] <- matrix(0, forecast_period, p)
-      for (i in 1:forecast_period) {
-        result[[paste0("Cycle", j)]][i, ] <- tempZ %*% matrix(a_fc[i, ])
-      }
+      tempZ_t <- t(tempZ)
+      result[[paste0("Cycle", j)]] <- a_fc %*% tempZ_t
       Z_padded[[paste0("Cycle", j)]] <- tempZ
     }
   }
@@ -315,10 +303,8 @@ predict.statespacer <- function(object,
     for (j in seq_along(object$function_call$arima_list)) {
       tempZ <- matrix(0, p, m)
       tempZ[1:length(Z_padded[[paste0("ARIMA", j)]])] <- Z_padded[[paste0("ARIMA", j)]]
-      result[[paste0("ARIMA", j)]] <- matrix(0, forecast_period, p)
-      for (i in 1:forecast_period) {
-        result[[paste0("ARIMA", j)]][i, ] <- tempZ %*% matrix(a_fc[i, ])
-      }
+      tempZ_t <- t(tempZ)
+      result[[paste0("ARIMA", j)]] <- a_fc %*% tempZ_t
       Z_padded[[paste0("ARIMA", j)]] <- tempZ
     }
   }
@@ -328,10 +314,8 @@ predict.statespacer <- function(object,
     for (j in seq_along(object$function_call$sarima_list)) {
       tempZ <- matrix(0, p, m)
       tempZ[1:length(Z_padded[[paste0("SARIMA", j)]])] <- Z_padded[[paste0("SARIMA", j)]]
-      result[[paste0("SARIMA", j)]] <- matrix(0, forecast_period, p)
-      for (i in 1:forecast_period) {
-        result[[paste0("SARIMA", j)]][i, ] <- tempZ %*% matrix(a_fc[i, ])
-      }
+      tempZ_t <- t(tempZ)
+      result[[paste0("SARIMA", j)]] <- a_fc %*% tempZ_t
       Z_padded[[paste0("SARIMA", j)]] <- tempZ
     }
   }
@@ -341,14 +325,12 @@ predict.statespacer <- function(object,
     if (is.matrix(Z_padded$self_spec)) {
       tempZ <- matrix(0, p, m)
       tempZ[1:length(Z_padded$self_spec)] <- Z_padded$self_spec
+      tempZ_t <- t(tempZ)
+      result$self_spec <- a_fc %*% tempZ_t
     } else {
       tempZ <- array(0, dim = c(p, m, forecast_period))
-    }
-    result$self_spec <- matrix(0, forecast_period, p)
-    for (i in 1:forecast_period) {
-      if (is.matrix(Z_padded$self_spec)) {
-        result$self_spec[i, ] <- tempZ %*% matrix(a_fc[i, ])
-      } else {
+      result$self_spec <- matrix(0, forecast_period, p)
+      for (i in 1:forecast_period) {
         tempZ[, , i][1:length(Z_padded$self_spec[, , i])] <- Z_padded$self_spec[, , i]
         result$self_spec[i, ] <- matrix(tempZ[, , i], nrow = p) %*% matrix(a_fc[i, ])
       }
